/*
   COM port connection library
   by paul69, v0.00, 2003-02-08, <e2500@narod.ru>

   Supported camera:
      (not tested)
*/
#define STRICT
#include <windows.h>
#pragma hdrstop

#include <stdio.h>

#include "dsc_connect.h"

class COM_DSC : public DSC
{

public:

   COM_DSC(void* _handle, const char* _comment) : DSC(_handle, _comment) {}
  ~COM_DSC();

   virtual bool SetReg(int reg, unsigned value);
   virtual bool GetReg(int reg, unsigned& value);

   virtual bool SetVar(int reg, const void* data, unsigned size);
   virtual bool SetStr(int reg, const char* str);
   virtual bool GetVar(int reg, TResponce&);

   virtual bool SpecCmd(int reg, unsigned value, unsigned timeout=0);

   virtual bool GetDeviceInfo(TDeviceInfo&);
   virtual bool CloseCamera();
};


////////////////////////////////////////////////////////////////////////////////
//
//
//
//
//

static word check_sum( const void* buf, int len )
{
   unsigned sum = 0;
   const byte* s = (const byte*)buf;

   while( len-- )
      sum += *s++;

   return (word)sum;
}

bool SendBytes(HANDLE hDSC, const void* buf, unsigned num)
{
   DWORD w_count = 0;
   if( !WriteFile( hDSC, buf, num, &w_count, NULL ) ) return false;
   if( w_count != num ) return false;
   if( !FlushFileBuffers( hDSC ) ) return false;
   return true;
}

inline bool SendByte(HANDLE hDSC, BYTE b) { return SendBytes(hDSC, &b, 1); }

int Ack(HANDLE hDSC, BYTE* ack)
{
   DWORD r_count = 0;
   if( !ReadFile( hDSC, ack, 1, &r_count, NULL ) ) return -1;
   return r_count;
}

bool COM_DSC::GetVar(int reg, TResponce& responce)
{
   responce.free();

   struct TCmd
   {
      byte command;
      word length;
   };

   struct TData
   {
      byte command;
      byte reg;
      word sum;
   };

   TCmd cmd;
   cmd.command = 0x43;
   cmd.length = 2;

   TData data;
   data.command = 4;
   data.reg = (byte)reg;
   data.sum = check_sum( &data, 2 );

   BYTE ack;
   if( ::Ack( GetHandle(), &ack ) < 0 ) return false;
   if( !::SendByte( GetHandle(), 0x1B ) ) return false;
   if( !::SendBytes( GetHandle(), &cmd, sizeof(cmd) ) ) return false;
   if( !::SendBytes( GetHandle(), &data, sizeof(data) ) ) return false;

   for(;;)
   {
      struct TResp
      {
         byte cmd;
         byte frame;
         word size;
      } resp;

      DWORD r_count;
      if( !::ReadFile( GetHandle(), &resp, sizeof(resp), &r_count, NULL ) ) return false;
      if( r_count != sizeof(resp) ) return false;

      byte buf[PACKET_SIZE + 8];

      if( !::ReadFile( GetHandle(), buf, resp.size, &r_count, NULL ) ) return false;
      if( r_count != resp.size ) return false;

      WORD sum;
      if( !::ReadFile( GetHandle(), &sum, 2, &r_count, NULL ) ) return false;
      if( r_count != 2 ) return false;

      if( !::SendByte( GetHandle(), 0x06 ) ) return false;

      if( resp.cmd != 3 && resp.cmd != 2 )
         return false;

      responce.append(buf, resp.size);

      if( resp.cmd == 3 )
         break;
   }
   return true;
}

bool COM_DSC::SetStr(int reg, const char* s)
{
   return SetVar(reg, s, strlen(s) );
}

bool COM_DSC::SetVar(int reg, const void* p, unsigned size)
{
   for(int i=0; size; ++i )
   {
      int sz;
      BYTE ack;

      if( i == 0 )
      {
         if( ::Ack( GetHandle(), &ack ) < 0 ) return false;
         if( !::SendByte( GetHandle(), 0x1B ) ) return false;

         struct TCmd
         {
            byte command;
            word length;
         };

         TCmd cmd;

         sz = size;
         if( sz > PACKET_SIZE-2 )
            sz = PACKET_SIZE-2;

         cmd.command = 0x43;
         cmd.length = (word)(sz+2);

         if( !::SendBytes( GetHandle(), &cmd, sizeof(cmd) ) ) return false;

         struct TData
         {
            byte command;        // -2 bytes in header, but +2 bytes in check_sum
            byte reg;
            byte data[PACKET_SIZE-2+2];
         } data;

         data.command = 3;
         data.reg = (byte)reg;
         memcpy( data.data, p, sz );

         word sum = ::check_sum( &data, sz+2 );
         data.data[sz] = (byte)(sum);
         data.data[sz+1] = (byte)(sum>>8);

         if( !::SendBytes( GetHandle(), &data, sz+4 ) ) return false;
      }
      else
      {
         struct TPacket
         {
            byte cmd;
            byte packet;
            word length;
         };

         TPacket packet;
         packet.cmd = 3;

         sz = size;
         if( sz > PACKET_SIZE )
         {
            sz = PACKET_SIZE;
            packet.cmd = 2;
         }

         packet.packet = (byte)(i-1);
         packet.length = (word)sz;

         if( !::SendBytes( GetHandle(), &packet, sizeof(packet) ) ) return false;

         struct TData
         {
            byte data[PACKET_SIZE+2];
         } data;

         memcpy( data.data, p, sz );
         word sum = ::check_sum( &data, sz );
         data.data[sz] = (byte)(sum);
         data.data[sz+1] = (byte)(sum>>8);

         if( !::SendBytes( GetHandle(), &data, sz+2 ) ) return false;
      }

      if( ::Ack( GetHandle(), &ack ) <= 0 || ack != 0x06 ) return false;

      if( (size -= sz) == 0 ) break;
      p = (const void*)((unsigned)p + sz);
   }
   return true;
}

bool COM_DSC::GetReg(int reg, unsigned& value)
{
   struct TCmd
   {
      byte command;
      word length;
   };

   struct TData
   {
      byte command;
      byte reg;
      word sum;
   };

   TCmd cmd;
   cmd.command = 0x43;
   cmd.length = 2;

   TData data;
   data.command = 1;
   data.reg = (byte)reg;
   data.sum = check_sum( &data, 2 );

   BYTE ack;
   if( ::Ack( GetHandle(), &ack ) < 0 ) return false;

   if( !::SendByte( GetHandle(), 0x1B ) ) return false;
   if( !::SendBytes( GetHandle(), &cmd, sizeof(cmd) ) ) return false;
   if( !::SendBytes( GetHandle(), &data, sizeof(data) ) ) return false;

   struct TResponce
   {
      byte code;
      byte packet;
      word len;
      dword value;
      word sum;
   };

   TResponce responce;

   DWORD r_count = 0;
   if( !::ReadFile( GetHandle(), &responce, sizeof(TResponce), &r_count, NULL ) ) return false;
   if( r_count != sizeof(TResponce) ) return false;

   if( !::SendByte( GetHandle(), 0x06 ) ) return false;
   
   value = responce.value;
   return true;
}

bool COM_DSC::SpecCmd(int reg, unsigned value, unsigned timeout)
{
   struct TCmd
   {
      byte command;
      word length;
   };

   struct TData
   {
      byte command;
      byte reg;
      byte value;
      word sum;
   };

   TCmd cmd;
   cmd.command = 0x43;
   cmd.length = 3;

   TData data;
   data.command = 2;
   data.reg = (byte)reg;
   data.value = (byte)value;
   data.sum = ::check_sum( &data, 3 );

   BYTE ack;
   if( ::Ack( GetHandle(), &ack ) < 0 ) return false;

   if( !::SendByte( GetHandle(), 0x1B ) ) return false;
   if( !::SendBytes( GetHandle(), &cmd, sizeof(cmd) ) ) return false;
   if( !::SendBytes( GetHandle(), &data, sizeof(data) ) ) return false;

   if( ::Ack( GetHandle(), &ack ) <= 0 || ack != 0x06 ) return false;

   if( timeout )
      ::Sleep( timeout );

   return true;
}

bool COM_DSC::SetReg(int reg, unsigned value)
{
   struct TCmd
   {
      byte command;
      word length;
   };

   struct TData
   {
      byte command;
      byte reg;
      dword value;
      word sum;
   };

   TCmd cmd;
   cmd.command = 0x43;
   cmd.length = 6;

   TData data;
   data.command = 0;
   data.reg = (byte)reg;
   data.value = value;
   data.sum = ::check_sum( &data, 6 );

   BYTE ack;
   if( ::Ack( GetHandle(), &ack ) < 0 ) return false;

   if( !::SendByte( GetHandle(), 0x1B ) ) return false;
   if( !::SendBytes( GetHandle(), &cmd, sizeof(cmd) ) ) return false;
   if( !::SendBytes( GetHandle(), &data, sizeof(data) ) ) return false;

   if( ::Ack( GetHandle(), &ack ) <= 0 || ack != 0x06 ) return false;

   return true;
}

bool SetSerialSpeed(HANDLE hDSC, unsigned speed)
{
   struct TCmd
   {
      byte command;
      word length;
   };

   struct TData
   {
      byte command;
      byte reg;
      dword value;
      word sum;
   };

   TCmd cmd;
   cmd.command = 0x53;
   cmd.length = 6;

   TData data;
   data.command = 0;
   data.reg = 0x11;
   data.value = speed;
   data.sum = ::check_sum( &data, 6 );

   BYTE ack;
   if( ::Ack( hDSC, &ack ) < 0 ) return false;

   if( !::SendByte( hDSC, 0x1B ) ) return false;
   if( !::SendBytes( hDSC, &cmd, sizeof(cmd) ) ) return false;
   if( !::SendBytes( hDSC, &data, sizeof(data) ) ) return false;

   if( ::Ack( hDSC, &ack ) <= 0 || ack != 0x06 ) return false;

   return true;
}

bool COM_DSC::GetDeviceInfo(TDeviceInfo& info)
{
   memset( &info, 0, sizeof(info) );

   TResponce responce;

   if( !GetVar(0x70, responce) )
      return false;

   struct _t {
      char VendorId[8];
      char ProductId[16];
      char ProductRevisionLevel[4];
      char SerialNumber[12];
   } *t = (_t*)responce.ptr;

   if( responce.size < 28 )
      return false;
      
   memcpy( info.VendorId, t->VendorId, sizeof(t->VendorId) );
   info.VendorId[sizeof(t->VendorId)] = '\0';

   memcpy( info.ProductId, t->ProductId, sizeof(t->ProductId) );
   info.VendorId[sizeof(t->ProductId)] = '\0';

   memcpy( info.ProductRevisionLevel, t->ProductRevisionLevel, sizeof(t->ProductRevisionLevel) );
   info.VendorId[sizeof(t->ProductRevisionLevel)] = '\0';
   return true;
}

bool COM_DSC::CloseCamera()
{
   // set default speed of the camera serial port
   return SetSerialSpeed( GetHandle(), 0);
}

COM_DSC::~COM_DSC()
{
   if( GetHandle() == INVALID_HANDLE_VALUE )
      return;

   CloseCamera();
}

bool CheckConnection(HANDLE hDSC)
{
   COMMTIMEOUTS ComTimeouts;
   DCB Dcb;

   memset( &Dcb, 0, sizeof( Dcb ) );
   memset( &ComTimeouts, 0, sizeof( ComTimeouts ) );

   if( !GetCommState( hDSC, &Dcb ) ) return false;

   Dcb.BaudRate = CBR_19200;
   Dcb.ByteSize = 8;
   Dcb.Parity = 0;
   Dcb.StopBits = 0;
   Dcb.fOutxCtsFlow = 0;
   Dcb.fOutxDsrFlow = 0;
   Dcb.fOutX = 0;
   Dcb.fInX = 0;

   if( !SetCommState( hDSC, &Dcb ) ) return false;

   if( !GetCommTimeouts( hDSC, &ComTimeouts ) ) return false;

   ComTimeouts.ReadIntervalTimeout = 0;
   ComTimeouts.ReadTotalTimeoutMultiplier = 0;
   ComTimeouts.ReadTotalTimeoutConstant = 3600;
   ComTimeouts.WriteTotalTimeoutMultiplier = 100;
   ComTimeouts.WriteTotalTimeoutConstant = 300;

   if( !SetCommTimeouts( hDSC, &ComTimeouts ) ) return false;

   if( !SetupComm( hDSC, 4096, 4096 ) ) return false;

   if( !SendByte( hDSC, 0 ) ) return false;

   BYTE ack;
   if( Ack( hDSC, &ack ) <= 0 || ack != 0x15 ) return false;

   // set high speed of the camera serial port
   if( !SetSerialSpeed(hDSC, 5) ) return false;

   Dcb.BaudRate = CBR_115200;
   if( !SetCommState( hDSC, &Dcb ) ) return false;

   return true;
}

static char DSC_COM[] = "COM0";

static HANDLE doOpenDSC(char comm_port)
{
   DSC_COM[3] = comm_port;

   HANDLE hDSC = CreateFile(DSC_COM, GENERIC_READ|GENERIC_WRITE, 0,
      NULL, OPEN_EXISTING, 0, NULL );

   if( hDSC != INVALID_HANDLE_VALUE )
   {
      if( CheckConnection(hDSC) )
         return hDSC;

      CloseHandle( hDSC );
   }
   return INVALID_HANDLE_VALUE;
}

DSC* OpenCOM(char comm_port)
{
   if( comm_port >= '0' && comm_port <= '9')
   {
      HANDLE hDSC = doOpenDSC( comm_port );
      if( hDSC == INVALID_HANDLE_VALUE )
         return NULL;

      return new COM_DSC( hDSC, DSC_COM );
   }

   for(int i=1; i < 4; ++i)
   {
      HANDLE hDSC = doOpenDSC( (char)('0'+i) );

      if( hDSC != INVALID_HANDLE_VALUE )
         return new COM_DSC( hDSC, DSC_COM );
   }
   return NULL;
}


