#define STRICT
#include <windows.h>
#pragma hdrstop

#include "rc.h"
#include "dsc_connect.h"

const char* ProgName = "Coolpix RAW Enabler";
DSC* dsc;
HINSTANCE hAppInst;
HWND hAppWin;

bool raw_enabled = false;
bool changed = false;

void InitDlg(HWND hDlg)
{
   SetDlgItemText(hDlg, IDC_MESSAGE, "Connecting to camera...");
   EnableWindow( GetDlgItem(hDlg, IDC_RAW_ENABLED), FALSE );
   EnableWindow( GetDlgItem(hDlg, IDOK), FALSE );

   PostMessage(hDlg, WM_COMMAND, IDC_CONNECT, 0);
}

bool GetRAWMode(HWND hDlg)
{
   if( !dsc ) return false;
   
   TResponce responce;

   if( dsc->GetVar(0x16, responce) )
   {
      raw_enabled = false;
      if( strstr(responce.GetStr(), "DIAG") != 0 && strstr(responce.GetStr(), "RAW") != 0 )
         raw_enabled = true;

      EnableWindow( GetDlgItem(hDlg, IDC_RAW_ENABLED), TRUE );

      CheckDlgButton(hDlg, IDC_RAW_ENABLED, raw_enabled ? BST_CHECKED:BST_UNCHECKED);
      return true;
   }
   return false;
}

bool Connect(HWND hDlg)
{
   TDeviceInfo info;
   char mess[80];

   dsc = OpenUSB();
   if( !dsc )
      dsc = OpenCOM();

   if( !dsc )
      return false;

   if( dsc->GetDeviceInfo( info ) )
   {
      wsprintf(mess, "%s %s%s", dsc->comment, info.ProductId, info.ProductRevisionLevel);
   }
   else
   {
      wsprintf(mess, "Camera connected via %s", dsc->comment );
   }

   SetDlgItemText(hDlg, IDC_MESSAGE, mess);
   return true;
}

bool Update()
{
   if( !dsc ) return false;
   return dsc->SetStr(0x16, raw_enabled ? "DIAG=RAW" : "NIKON DIGITAL CAMERA" );
}

BOOL Command(HWND hDlg, int NotifyCode, int id, HWND hCtrl)
{
   NotifyCode;
   hCtrl;

   switch(id)
   {
   case IDCANCEL:
      EndDialog(hDlg, 0);
      return TRUE;

   case IDOK:
      // Update
      if( changed )
      {
         if( Update() )
            MessageBox(hDlg, raw_enabled ?
               "RAW mode enabled":"RAW mode disabled", ProgName, MB_ICONINFORMATION|MB_OK);
         else
            MessageBox(hDlg, "Cannot change RAW mode", ProgName, MB_ICONSTOP|MB_OK);
      }
      EndDialog(hDlg, 1);
      return TRUE;

   case IDC_RAW_ENABLED:

      if( !changed )
         EnableWindow( GetDlgItem(hDlg, IDOK), TRUE );

      changed = true;
      raw_enabled = ! raw_enabled;

      CheckDlgButton(hDlg, IDC_RAW_ENABLED, raw_enabled ? BST_CHECKED:BST_UNCHECKED);
      return TRUE;

   case IDC_CONNECT:

      if( !Connect(hDlg) )
      {
         SetDlgItemText(hDlg, IDC_MESSAGE, "Camera is not found");

         MessageBox(hDlg,
            "Can not connect to camera.\r\n"
            "Check USB or COM connection \r\n"
            "and try again.", ProgName, MB_OK|MB_ICONSTOP);

         EndDialog(hDlg, 0);
         return TRUE;
      }

      if( !GetRAWMode(hDlg) )
         GetRAWMode(hDlg);
         
      return TRUE;

   default:
      return FALSE;
   }
}

BOOL CALLBACK MainDlg( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   if( uMsg == WM_INITDIALOG )
   {
      if( !hAppWin )
         hAppWin = hDlg;

      InitDlg(hDlg);
      return TRUE;
   }

   if( uMsg == WM_COMMAND )
   {
      return Command(hDlg, HIWORD(wParam), LOWORD(wParam), (HWND) lParam);
   }

   if( uMsg == WM_CLOSE )
   {
      EndDialog(hDlg, 0);
      return TRUE;
   }

   if( uMsg == WM_DESTROY )
   {
      if( hDlg == hAppWin )
         hAppWin = NULL;

      return FALSE;
   }
   return FALSE;
}

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, int )
{
   hAppInst = hInstance;

   DialogBox( hAppInst, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDlg);

   delete dsc;
   dsc = NULL;
   return 0;
}

